PYTHON2_VERSION = (2, 7)
PYTHON3_VERSION = (3, 6)
PYTHON3_VERSION_MIN = (3, 3)
CACHE_DIR = '.mypy_cache'
CONFIG_FILE = 'mypy.ini'
SHARED_CONFIG_FILES = ('setup.cfg',)
USER_CONFIG_FILES = ('~/.mypy.ini',)
CONFIG_FILES = (CONFIG_FILE,) + SHARED_CONFIG_FILES + USER_CONFIG_FILES
