from typing import Any

Cc = ...  # type: str
Cf = ...  # type: str
Cn = ...  # type: str
Co = ...  # type: str
Cs = ...  # type: Any
Ll = ...  # type: str
Lm = ...  # type: str
Lo = ...  # type: str
Lt = ...  # type: str
Lu = ...  # type: str
Mc = ...  # type: str
Me = ...  # type: str
Mn = ...  # type: str
Nd = ...  # type: str
Nl = ...  # type: str
No = ...  # type: str
Pc = ...  # type: str
Pd = ...  # type: str
Pe = ...  # type: str
Pf = ...  # type: str
Pi = ...  # type: str
Po = ...  # type: str
Ps = ...  # type: str
Sc = ...  # type: str
Sk = ...  # type: str
Sm = ...  # type: str
So = ...  # type: str
Zl = ...  # type: str
Zp = ...  # type: str
Zs = ...  # type: str
cats = ...  # type: Any

def combine(*args): ...

xid_start = ...  # type: str
xid_continue = ...  # type: str

def allexcept(*args): ...
