"""Stub file for the '_tracemalloc' module."""
# This is an autogenerated file. It serves as a starting point
# for a more precise manual annotation of this module.
# Feel free to edit the source below, but remove this header when you do.

from typing import Any, List, Tuple, Dict, Generic

def _get_object_traceback(*args, **kwargs) -> Any: ...

def _get_traces() -> Any:
    raise MemoryError()

def clear_traces() -> None: ...

def get_traceback_limit() -> int: ...

def get_traced_memory() -> tuple: ...

def get_tracemalloc_memory() -> Any: ...

def is_tracing() -> bool: ...

def start(*args, **kwargs) -> None:
    raise ValueError()

def stop() -> None: ...
